/*
    AFFcleaner - removes Adaptation Field fillers in MPEG-TS recordings
    Copyright (C) 2015 Stefan Pöschel

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stdio.h>
#include <stdint.h>
#include <string.h>
#include <inttypes.h>

#define PKT_LEN 188

uint64_t bytes_read = 0;
uint64_t bytes_written = 0;
uint64_t bytes_removed = 0;
uint64_t bytes_removed_pid[0x1FFF] = {0};


void print_progress() {
	double percent_removed = (double) bytes_removed / (double) bytes_read * 100.0;
	printf("\rBytes: %" PRIu64 " read, %" PRIu64 " written, %" PRIu64 " removed (%.1f%%)",
			bytes_read, bytes_written, bytes_removed, percent_removed);
}

int main(int argc, char** argv) {
	FILE *in_file = NULL;
	FILE *out_file = NULL;
	unsigned char pkt[PKT_LEN];
	unsigned char af[184];

	printf("AFFcleaner v1.0a - Copyright Stefan Poeschel 2015\n");
	printf("\n");

	if(argc != 3) {
		printf("Removes Adaptation Field fillers in MPEG-TS recordings\n");
		printf("\n");
		printf("Usage: %s <in_file> <out_file>\n", argv[0]);
		return 1;
	}

	printf("Reading: %s\n", argv[1]);
	printf("Writing: %s\n", argv[2]);
	printf("\n");

	in_file = fopen(argv[1], "rb");
	if(!in_file) {
		perror("Error while opening input file");
		return 1;
	}

	out_file = fopen(argv[2], "wb");
	if(!out_file) {
		fclose(in_file);
		perror("Error while opening output file");
		return 1;
	}


	/* the special Adaptation Field consists of:
	 * - adaptation_field_length
	 * - indicator/flag bitmask
	 * - stuffing bytes
	 */
	af[0] = 0xB7;
	af[1] = 0x00;
	memset(af + 2, 0xFF, sizeof(af) - 2);

	while(1) {
		if((bytes_read / PKT_LEN) % 10000 == 0)
			print_progress();

		if(fread(pkt, PKT_LEN, 1, in_file) != 1) {
			if(feof(in_file))
				print_progress();
			else
				perror("\nError reading packet");
			break;
		}

		if(pkt[0] != 0x47) {
			printf("\nNo sync at offset %" PRIu64 " - abort!\n", bytes_read);
			break;
		}

		bytes_read += PKT_LEN;

		/* skip packet, if not Null Packet and (only) Adaptation Field filler present */
        int pid = (pkt[1] & 0x1F) << 8 | pkt[2];
		if(pid != 0x1FFF &&
		        ((pkt[3] & 0x30) >> 4 == 0b10) &&
				!memcmp(pkt + 4, af, sizeof(af))) {
			bytes_removed += PKT_LEN;
			bytes_removed_pid[pid] += PKT_LEN;
		} else {
			if(fwrite(pkt, PKT_LEN, 1, out_file) != 1) {
				perror("\nError writing packet");
				break;
			}
			bytes_written += PKT_LEN;
		}
	}

	printf("\n\n");

    int pid;
    for(pid = 0; pid < 0x1FFF; pid++)
        if(bytes_removed_pid[pid])
            printf("PID %4d: %11" PRId64 " bytes\n", pid, bytes_removed_pid[pid]);

	fclose(in_file);
	fclose(out_file);

	return 0;
}
